<?php
namespace Teknisa\DF\Modules\Relatorios\Services;

use Teknisa\Libs\Util\DateUtil;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Adm03000ReDeNotasFiscais{

    protected $crudService;
    protected $parameterBag;
    protected $dataSourceManager;
    protected $Adm03000Factory;
    protected $filial;
    protected $nrorg;
    protected $parameterBagColumns;
    protected $dataSourceName;

    public function __construct($crudService, $parameterBag, $dataSourceManager, $environment, $Adm03000Factory) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager =$dataSourceManager;
        $this->filial = $environment->getCdFilial();
        $this->nrorg = $environment->getNrOrgTrab();
        $this->Adm03000Factory = $Adm03000Factory;
    }

    public function verificaNotaFiscal($row, $filial) {
        if($filial === 'FILIAL_LOGADA'){
            $filial =  $this->filial;
        }

        $conditions[] = array(
           'columnName' => 'CDFILIAL',
           'operator' => '=',
           'value' => $filial
        );
        $conditions[] = array(
            'columnName' => 'NRORG',
            'operator' => '=',
            'value' => $this->nrorg
        );

        $dataSet = $this->crudService->getFilteredDataSet('paramestoque_processamento', $conditions);
        $rows = $dataSet->getRows();
        if(!empty($rows)){
            $data = $rows[0]['DTPROCESSA'];
            $data = substr($data, 0, 10);

            $this->dataSourceName = 'notafiscal_forAdm03000';
            $this->parameterBagColumns = array(
                "CDFILIAL",
                "CDALMOXARIFE",
                "DATARE",
                "CDLOCALESTOQ"
            );

            if ($row['IDUTILALMOX'] === 'S'){
                $cdalmoxarife = null;
            } else {
                $cdalmoxarife = ' ';
            }

            if ($row['IDUTILLCESTQ'] === 'S'){
                $cdlocalestoq = null;
            } else {
                $cdlocalestoq = ' ';
            }

            $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria($this->dataSourceName);
            $filterCriteria->addCondition('CDFILIAL',$filial);
            $filterCriteria->addCondition('DATARE',$data);
            $filterCriteria->addCondition('CDALMOXARIFE',$cdalmoxarife);
            $filterCriteria->addCondition('CDLOCALESTOQ',$cdlocalestoq);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
            $newRow = $dataSet->getRows();
            if(empty($newRow)){
                $newRow[0]['NRCOUNTCGC'] = null;
                $newRow[0]['NRCOUNTEMIS'] = null;
                $newRow[0]['VRTOTANOTA'] = null;
                $newRow[0]['NRCOUNTNUM'] = null;
            }
            $newRow[0]['DATARE'] = $data;
            return $newRow[0];
        } else {
            throw new \Exception("Erro ao verificar se a filial utiliza ou não data de processamento.", 1);
        }
    }

    public function verificaNumeracaoRe($cdfilial){
        $nrRe = $this->Adm03000Factory->verificaNumeracaoRe($cdfilial, $this->nrorg);
        if(empty($nrRe)){
            throw new \Exception("Operação Bloqueada! Sequencial do RE deve ser parametrizado para a filial.", 1);
        }
    }

    public function verificaCapaDocumento($centraliza, $cdalmoxarife, $datare, $cdlocalestoq, $cdfilial, $identrsaid){
        if($centraliza === 'F'){
            $checaDocs = $this->Adm03000Factory->verificaCapaDocumentoNaoCentralia($cdalmoxarife, $datare, $cdlocalestoq, $cdfilial, $identrsaid, $this->nrorg);
            if(!empty($checaDocs)){
                $ReDataInformada = $this->Adm03000Factory->reDataInformada($cdfilial, $datare, $identrsaid, $this->nrorg);
                $ReDataPosterior = $this->Adm03000Factory->reDataPosterior($cdfilial, $datare, $identrsaid, $this->nrorg);
                if(!empty($ReDataInformada) || (empty($ReDataInformada) && empty($ReDataPosterior))){
                    $dataAntRe = DateUtil::getDataDeString($datare);
                    $dataAntRe = DateUtil::subtraiIntervalo($dataAntRe, 1);
                    $dataAntRe = $dataAntRe->format('d/m/Y');
                    $dtAnterior = substr($datare, 6, 9);
                    $dtAnterior = "01/01/$dtAnterior";
                    $IDNIRREG = $this->notCentralizadora($cdalmoxarife, $datare, $cdlocalestoq, $cdfilial, $identrsaid, $dataAntRe, $dtAnterior);
                    $this->geraProcedure($IDNIRREG, $cdfilial, $identrsaid, $datare, 'F', $cdalmoxarife, $cdlocalestoq, true);
                } else {
                    throw new \Exception("Operação bloqueada! Para a data informada não possui capa de documento(RE) e existem capa(s) de documento(RE) posteriores.", 1);
                }
            } else {
                throw new \Exception("Não há dados a serem listados.",1);
            }
        } else {
            $checaDocs = $this->Adm03000Factory->verificaCapaDocumentoCentrliza($cdalmoxarife, $datare, $cdlocalestoq, $cdfilial, $this->nrorg);
            if(!empty($checaDocs)){
                $ReDataInformada = $this->Adm03000Factory->reDataInformada($cdfilial, $datare, $identrsaid, $this->nrorg);
                $ReDataPosterior = $this->Adm03000Factory->reDataPosterior($cdfilial, $datare, $identrsaid, $this->nrorg);
                if(!empty($ReDataInformada) || (empty($ReDataInformada) && empty($ReDataPosterior))){
                    $dataAntRe = DateUtil::getDataDeString($datare);
                    $dataAntRe = DateUtil::subtraiIntervalo($dataAntRe, 1);
                    $dataAntRe = $dataAntRe->format('d/m/Y');
                    $dtAnterior = substr($datare, 6, 9);
                    $dtAnterior = "01/01/$dtAnterior";
                    $IDNIRREG = $this->centraliza($cdalmoxarife, $datare, $cdlocalestoq, $cdfilial, $identrsaid, $dataAntRe, $dtAnterior);
                    $this->geraProcedure($IDNIRREG, $cdfilial, $identrsaid, $datare, 'C', $cdalmoxarife, $cdlocalestoq, true);
                } else {
                    throw new \Exception("Operação bloqueada! Para a data informada não possui capa de documento(RE) e existem capa(s) de documento(RE) posteriores.", 1);
                }
            } else {
                throw new \Exception("Não há dados a serem listados.",1);
            }
        }
    }

    public function notCentralizadora($cdalmoxarife, $datare, $cdlocalestoq, $cdfilial, $identrsaid, $dataAntRe, $dtAnterior){
        if($identrsaid === 'E'){
            if(!empty($this->Adm03000Factory->ferivicaReAnoCor($cdfilial, $dataAntRe, $dtAnterior, $this->nrorg))){
                $reAntesCapa = $this->Adm03000Factory->reAnteSCapaE($cdfilial, $identrsaid, $dataAntRe, $dtAnterior, $this->nrorg);
                if(!empty($reAntesCapa) && $reAntesCapa[0]['DTNUMERNF'] !== null){
                    $dtnumernf = substr($reAntesCapa[0]['DTNUMERNF'], 0, 10);
                    $verificaRe = $this->Adm03000Factory->ferivicaRe($cdfilial, $identrsaid, $dtnumernf, $this->nrorg);
                    if(empty($verificaRe)){
                        return 'N';
                    } else {
                        return 'S';
                    }
                } else {
                    return 'S';
                }
            } else {
                return 'S';
            }
        } else {
            if(!empty($this->Adm03000Factory->ferivicaReAnoCor($cdfilial, $dataAntRe, $dtAnterior, $this->nrorg))){
                $reAntesCapa = $this->Adm03000Factory->reAnteSCapaS($cdfilial, $identrsaid, $dataAntRe, $dtAnterior, $this->nrorg);
                if(!empty($reAntesCapa) && $reAntesCapa[0]['DTNUMERNF'] !== null){
                    $dtnumernf = substr($reAntesCapa[0]['DTNUMERNF'], 0, 10);
                    $verificaRe = $this->Adm03000Factory->ferivicaRe($cdfilial, $identrsaid, $dtnumernf, $this->nrorg);
                    if(empty($verificaRe)){
                        return 'N';
                    } else {
                        return 'S';
                    }
                } else {
                    return 'S';
                }
            } else {
                return 'S';
            }
        }
    }

    public function centraliza($cdalmoxarife, $datare, $cdlocalestoq, $cdfilial, $identrsaid, $dataAntRe, $dtAnterior){
        if(!empty($this->Adm03000Factory->ferivicaReAnoCor($cdfilial, $dataAntRe, $dtAnterior, $this->nrorg))){
            $reAntesCapa = $this->Adm03000Factory->reAnteSCapaC($cdfilial, $identrsaid, $dataAntRe, $dtAnterior, $this->nrorg);
            if(!empty($reAntesCapa) && $reAntesCapa[0]['DTNUMERNF'] !== null){
                $dtnumernf = substr($reAntesCapa[0]['DTNUMERNF'], 0, 10);
                $verificaRe = $this->Adm03000Factory->ferivicaRe($cdfilial, $identrsaid, $dtnumernf, $this->nrorg);
                if(empty($verificaRe)){
                    return 'N';
                } else {
                    return 'S';
                }
            } else {
                return 'S';
            }
        } else {
            return 'S';
        }
    }

    public function geraProcedure($idnirreg, $cdfilial, $identrsaid, $datare, $tipoexpre, $cdalmoxarife, $cdlocalestoq, $iderro){
        if($idnirreg === 'S'){
            $this->Adm03000Factory->procedureGERA_RE($cdfilial, $identrsaid, $datare, $tipoexpre, $cdalmoxarife, $cdlocalestoq, $iderro);
        } else {
            throw new \Exception("Operação Bloqueada! Existem documentos anteriores sem capa de RE.", 1);
        }
    }
}
