<?php

namespace Teknisa\DF\Modules\Saida\NFSe;

use Teknisa\DF\Modules\Saida\NFSe\Utilities\Util;
use Teknisa\Libs\Util\Environment;

class NFSeCuiaba {
    const URL_PRODUCAO = "https://www.issnetonline.com.br/webserviceabrasf/cuiaba/servicos.asmx?wsdl";
    const URL_HOMOLOGACAO = "http://www.issnetonline.com.br/homologaabrasf/cuiaba/servicos.asmx?wsdl";

    protected $connection;
    protected $environment;
    protected $newCode;

    protected $ambiente; //1 = producao, 2 = homologacao

    // Dados para identificar o Certificado Digital
    protected $cdempresa;
    protected $cdinscmuni;

    protected $xmlEnviarLoteRps;
    protected $listaRps = array();
    protected $listaAuxRps = array();
    protected $xmlConsultarLoteRps;

    protected $publicKey;
    protected $privateKeyId;
    protected $caminhoCertificado;
    protected $linkCertificado;
    protected $linkCertKey;
    protected $caminhoXml;

    protected $senhaCertificadoDigital;

    // Dados Prestador
    protected $InscricaoPrestador;
    protected $InscricaoMunicipalPrestador;
    protected $RazaoSocialPrestador;
    protected $CodigoMunicipioPrestador;

    // Dados Lote RPS
    protected $IdLote;
	protected $NumeroLote;

	// Dados RPS
    protected $IdRps;
	protected $NumeroRps;
	protected $SerieRps;
	protected $TipoRps;
	protected $DataEmissao;
	protected $NaturezaOperacao;
	protected $RegimeEspecialTributacao;
	protected $OptanteSimplesNacional;
	protected $IncentivadorCultural;
	protected $StatusRps;
	protected $ValorServicos;
	protected $ValorDeducoes;
	protected $ValorPis;
	protected $ValorCofins;
	protected $ValorInss;
	protected $ValorIr;
	protected $ValorCsll;
	protected $IssRetido;
	protected $ValorIss;
	protected $ValorIssRetido;
	protected $OutrasRetencoes;
	protected $BaseCalculo;
	protected $Aliquota;
	protected $ValorLiquidoNfse;
	protected $DescontoCondicionado;
	protected $DescontoIncondicionado;
	protected $ItemListaServico;
	protected $CodigoCnae;
	protected $CodigoTributacaoMunicipio;
	protected $Discriminacao;
	protected $CodigoMunicipioTomador;
	protected $InscricaoTomador;
	protected $InscricaoMunicipalTomador;
	protected $RazaoSocialTomador;
	protected $Endereco;
	protected $Numero;
	protected $Complemento;
	protected $Bairro;
	protected $Uf;
	protected $Cep;
	protected $Telefone;
	protected $Email;

	public function __construct($connection, Environment $environment, $newCode, $xmlDir, $certUploadDir = null, $certUploadLink = null, $certKeyLink = null) {
        $this->connection = $connection;
        $this->environment = $environment;
        $this->newCode = $newCode;

        //$this->setCaminhoXml($xmlDir);
        if($certUploadDir) {
            $this->setCaminhoCertificado($certUploadDir);
        }
        else {
            $this->linkCertificado = $certUploadLink;
            $this->linkCertKey     = $certKeyLink;
        }
	}

	private function setCaminhoXml($xmlDir) {
	    $xmlDir .= "org_". $this->environment->getNrOrg() . DIRECTORY_SEPARATOR;
	    $this->caminhoXml = $xmlDir;
	    if (!is_dir($this->caminhoXml)) {
            mkdir($this->caminhoXml, 0775, true);
            chmod($this->caminhoXml, 0775);
        }
	}

	private function setCaminhoCertificado($certUploadDir) {
	    if (!is_null($certUploadDir)) {
	        $certUploadDir .= "org_". $this->environment->getNrOrg() . DIRECTORY_SEPARATOR;

	        $this->caminhoCertificado = $certUploadDir;
	        if (!is_dir($this->caminhoCertificado)) {
	            mkdir($this->caminhoCertificado, 0775, true);
	            chmod($this->caminhoCertificado, 0775);
	        }
	    }
	}
//'.$this->NumeroLote.'
	private function gerarXmlEnviarLoteRps() {
		$this->xmlEnviarLoteRps = '<EnviarLoteRpsEnvio xmlns="http://www.issnetonline.com.br/webserviceabrasf/vsd/servico_enviar_lote_rps_envio.xsd" 
		    xmlns:tc="http://www.issnetonline.com.br/webserviceabrasf/vsd/tipos_complexos.xsd">
		
			<LoteRps>
				<tc:NumeroLote>1</tc:NumeroLote>
				
				<tc:CpfCnpj>
					';
    				if (strlen($this->InscricaoPrestador) == 11) {
    					$this->xmlEnviarLoteRps .='<tc:Cpf>'.$this->InscricaoPrestador.'</tc:Cpf>';
    				}
    				else {
    					$this->xmlEnviarLoteRps .= '<tc:Cnpj>'.$this->InscricaoPrestador.'</tc:Cnpj>';
    				}
    				$this->xmlEnviarLoteRps .= '
				</tc:CpfCnpj>
				
				<tc:InscricaoMunicipal>'.$this->InscricaoMunicipalPrestador.'</tc:InscricaoMunicipal>
				<tc:QuantidadeRps>'.count($this->listaRps).'</tc:QuantidadeRps>
				<tc:ListaRps>';

				foreach ($this->listaRps as $rps) {
                    $this->xmlEnviarLoteRps .= "\n" . $rps;
                }

                $this->xmlEnviarLoteRps .= '
                </tc:ListaRps>
			</LoteRps>
		</EnviarLoteRpsEnvio>';
	}
//'.$this->NumeroRps.' '.$this->SerieRps.'  '.$this->NaturezaOperacao.''
//'.$this->Aliquota.'
	private function adicionarRpsEnviarLote() {
		$xmlRps = '
				<tc:Rps>
					<tc:InfRps>
						<tc:IdentificacaoRps>
							<tc:Numero>'.$this->NumeroRps.'</tc:Numero>
							<tc:Serie>'.$this->SerieRps.'</tc:Serie>
							<tc:Tipo>'.$this->TipoRps.'</tc:Tipo>
						</tc:IdentificacaoRps>
						<tc:DataEmissao>'.$this->DataEmissao.'</tc:DataEmissao>
						<tc:NaturezaOperacao>1</tc:NaturezaOperacao>
						<tc:OptanteSimplesNacional>'.$this->OptanteSimplesNacional.'</tc:OptanteSimplesNacional>
						<tc:IncentivadorCultural>'.$this->IncentivadorCultural.'</tc:IncentivadorCultural>
						<tc:Status>'.$this->StatusRps.'</tc:Status>
						<tc:RegimeEspecialTributacao>'.$this->RegimeEspecialTributacao.'</tc:RegimeEspecialTributacao>
						<tc:Servico>
							<tc:Valores>
								<tc:ValorServicos>'.Util::formatarNumero($this->ValorServicos).'</tc:ValorServicos>
								<tc:ValorPis>'.Util::formatarNumero($this->ValorPis).'</tc:ValorPis>
								<tc:ValorCofins>'.Util::formatarNumero($this->ValorCofins).'</tc:ValorCofins>
								<tc:ValorInss>'.Util::formatarNumero($this->ValorInss).'</tc:ValorInss>
								<tc:ValorIr>'.Util::formatarNumero($this->ValorIr).'</tc:ValorIr>
								<tc:ValorCsll>'.Util::formatarNumero($this->ValorCsll).'</tc:ValorCsll>
								<tc:IssRetido>'.$this->IssRetido.'</tc:IssRetido>
								<tc:ValorIss>'.Util::formatarNumero($this->ValorIss).'</tc:ValorIss>
								<tc:BaseCalculo>'.Util::formatarNumero($this->BaseCalculo).'</tc:BaseCalculo>
								<tc:Aliquota>'.$this->Aliquota.'</tc:Aliquota>
								<tc:ValorLiquidoNfse>'.Util::formatarNumero($this->ValorLiquidoNfse).'</tc:ValorLiquidoNfse>
								<tc:DescontoIncondicionado>'.Util::formatarNumero($this->DescontoIncondicionado).'</tc:DescontoIncondicionado>
								<tc:DescontoCondicionado>'.Util::formatarNumero($this->DescontoCondicionado).'</tc:DescontoCondicionado>
							</tc:Valores>
							<tc:ItemListaServico>'.$this->ItemListaServico.'</tc:ItemListaServico>
							<tc:CodigoCnae>'.$this->CodigoCnae.'</tc:CodigoCnae>
							<tc:CodigoTributacaoMunicipio>'.$this->CodigoTributacaoMunicipio.'</tc:CodigoTributacaoMunicipio>
							<tc:Discriminacao>'.$this->Discriminacao.'</tc:Discriminacao>
							<tc:MunicipioPrestacaoServico>'.$this->CodigoMunicipioPrestador.'</tc:MunicipioPrestacaoServico>
						</tc:Servico>
						<tc:Prestador>
						    <tc:CpfCnpj>
            					';
                				if (strlen($this->InscricaoPrestador) == 11) {
                					$xmlRps .= '<tc:Cpf>'.$this->InscricaoPrestador.'</tc:Cpf>';
                				}
                				else {
                					$xmlRps .=  '<tc:Cnpj>'.$this->InscricaoPrestador.'</tc:Cnpj>';
                				}
                				$xmlRps .=  '
            				</tc:CpfCnpj>

							<tc:InscricaoMunicipal>'.$this->InscricaoMunicipalPrestador.'</tc:InscricaoMunicipal>
						</tc:Prestador>
						<tc:Tomador>
							<tc:IdentificacaoTomador>
								<tc:CpfCnpj>
									';

								if (strlen($this->InscricaoTomador) == 11) {
									$xmlRps .= '<tc:Cpf>'.$this->InscricaoTomador.'</tc:Cpf>';
								}
								else {
									$xmlRps .= '<tc:Cnpj>'.$this->InscricaoTomador.'</tc:Cnpj>';
								}

								$xmlRps .= '
								</tc:CpfCnpj>
								<tc:InscricaoMunicipal>'.$this->InscricaoMunicipalTomador.'</tc:InscricaoMunicipal>
							</tc:IdentificacaoTomador>
							<tc:RazaoSocial>'.$this->RazaoSocialTomador.'</tc:RazaoSocial>
							<tc:Endereco>
								<tc:Endereco>'.$this->Endereco.'</tc:Endereco>
								<tc:Numero>'.$this->Numero.'</tc:Numero>
								<tc:Complemento>'.$this->Complemento.'</tc:Complemento>
								<tc:Bairro>'.$this->Bairro.'</tc:Bairro>';
								if (strlen($this->InscricaoTomador) != 11) {
								   $xmlRps .= ' <tc:Cidade>'.$this->CodigoMunicipioTomador.'</tc:Cidade>';
								}
								$xmlRps .= '
								<tc:Estado>'.$this->Uf.'</tc:Estado>
								<tc:Cep>'.$this->Cep.'</tc:Cep>
							</tc:Endereco>
						</tc:Tomador>
					</tc:InfRps>
				</tc:Rps>';

        /*
        valores

        <ValorIssRetido>'.Util::formatarNumero($this->ValorIssRetido).'</tc:ValorIssRetido>
        <DescontoCondicionado>'.Util::formatarNumero($this->DescontoCondicionado).'</tc:DescontoCondicionado>
        <DescontoIncondicionado>'.Util::formatarNumero($this->DescontoIncondicionado).'</tc:DescontoIncondicionado>
        */

        $this->listaRps[] = $xmlRps;
	}

	private function getXmlAssinadoEnviarLoteRps() {
	    $this->xmlEnviarLoteRps = str_replace("&", "&#38;", $this->xmlEnviarLoteRps);
		$domXml = new \DOMDocument("1.0", "UTF-8");
		$domXml->loadXML($this->xmlEnviarLoteRps, LIBXML_NOBLANKS);
        $domXml = $this->removerTagsVazias($domXml);
        $domXml->formatOutput = true;
        $xmlTemp = $domXml->saveXML($domXml->documentElement);

        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($xmlTemp);
        $domXml->encoding = "UTF-8";
        $domXml = $this->assinaXml($domXml, array("LoteRps"));
        if ($domXml === false) {
            throw new \Exception("Certificado digital não encontrado!");
        }
        else {
            $this->limparListaRps();
            return $domXml->saveXML($domXml->documentElement);
        }
	}

    private function removerTagsVazias(\DOMDocument $xml) {
        $xpath = new \DOMXPath($xml);
        foreach($xpath->query('//*[not(node())]') as $node) {
            if (!$node->hasAttributes()) {
                $node->parentNode->removeChild($node);
            }
        }
        return $xml;
    }

    private function limparListaRps() {
        $this->listaRps = array();
    }

    private function enviarLoteRpsWS($xml, $cdempresa, $inscricao, $numeroLote) {
        $nomeFuncao = "RecepcionarLoteRps";

        $prefixoArquivo = $cdempresa.$inscricao.$numeroLote."_".$nomeFuncao."_";
        $nomeArquivoEnvio = $prefixoArquivo."env.xml";
        //file_put_contents($this->caminhoXml . $nomeArquivoEnvio, $xml);

        try {
            $resultado = $this->enviarParaWebservice($nomeFuncao, $xml);
        }
        catch (\Exception $ex) {
            throw new \Exception($ex->getMessage());
        }

        if ($resultado->outputXML) {
            $nomeArquivoResposta = $prefixoArquivo."resp.xml";
            //file_put_contents($this->caminhoXml . $nomeArquivoResposta, $resultado->outputXML);

            $domResult = new \DOMDocument("1.0", "UTF-8");
            $domResult->loadXML($resultado->outputXML);

            if ($domResult->getElementsByTagName("NumeroLote")->length > 0) {

                $protocolo = $domResult->getElementsByTagName("Protocolo")->item(0)->nodeValue;

                foreach ($this->listaAuxRps as $nota) {
                    Util::atualizarNotaTransmissao($this->connection, $nota["CDSERIERPS"], $nota["NRRPS"], $nota["CDEMPLOTERPS"], $nota["CDINSCESTA"], $nota["CDINSCMUNI"], $nota["NRSEQLOTERPS"], $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
                }

                Util::atualizarLote($this->connection, $this->listaAuxRps[0]["CDEMPLOTERPS"], $this->listaAuxRps[0]["CDINSCESTA"], $this->listaAuxRps[0]["CDINSCMUNI"], $this->NumeroLote, 2, $protocolo, $this->environment->getNrOrg(), $this->environment->getCdOperador());

                return true;
            }
            else if ($domResult->getElementsByTagName("MensagemRetorno")->length > 0) {
                $erros = array("TIPO" => 2);
                $erros["XML"] = $xml;

                foreach ($domResult->getElementsByTagName("MensagemRetorno") as $erro) {
                    $erros[] = array(
                        "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                        "DESCRICAO" => Util::getChildNodeByName($erro, "Mensagem")->nodeValue
                    );
                }
                
                file_put_contents("/tmp/".$nomeArquivoEnvio, $xml);
                return $erros;
            }
        }
        else {
            return false;
        }
    }

    private function consultarLoteRpsWS($xml, $cdempresa, $inscricao, $numeroLote) {
        $nomeFuncao = "ConsultarLoteRps";
        $prefixoArquivo = $cdempresa.$inscricao.$numeroLote."_".$nomeFuncao."_";
        $nomeArquivoEnvio = $prefixoArquivo."env.xml";
        //file_put_contents($this->caminhoXml . $nomeArquivoEnvio, $xml);

        try {
            $resultado = $this->enviarParaWebservice($nomeFuncao, $xml);
        }
        catch (\Exception $ex) {
            throw new \Exception($ex->getMessage());
        }

        $nomeArquivoResposta = $prefixoArquivo."resp.xml";
        if ($resultado->outputXML) {
            //file_put_contents($this->caminhoXml . $nomeArquivoResposta, $resultado->outputXML);
            return $resultado->outputXML;
        }
        else {
            return false;
        }
    }
    
    private function consultarSituacaoLoteRpsWS($xml, $cdempresa, $inscricao, $numeroLote) {
        $nomeFuncao = "ConsultarSituacaoLoteRPS";
        $prefixoArquivo = $cdempresa.$inscricao.$numeroLote."_".$nomeFuncao."_";
        $nomeArquivoEnvio = $prefixoArquivo."env.xml";
        //file_put_contents($this->caminhoXml . $nomeArquivoEnvio, $xml);

        try {
            $resultado = $this->enviarParaWebservice($nomeFuncao, $xml);
        }
        catch (\Exception $ex) {
            throw new \Exception($ex->getMessage());
        }

        $nomeArquivoResposta = $prefixoArquivo."resp.xml";
        
        if ($resultado->outputXML) {
            $domResult = new \DOMDocument("1.0", "UTF-8");
            $domResult->loadXML($resultado->outputXML);
            if ($domResult->getElementsByTagName("ConsultarSituacaoLoteRpsResposta")->length > 0) {
                $node = $domResult->getElementsByTagName("ConsultarSituacaoLoteRpsResposta");
                $status =  Util::getChildNodeByName($node[0], "Situacao");
                if($status != null) {
                    return $status->nodeValue;
                }
                else {
                    $listaMensagem   =  Util::getChildNodeByName($node[0], "ListaMensagemRetorno");
                    $mensagemRetorno = Util::getChildNodeByName($listaMensagem, "MensagemRetorno");
                    $mensagem        = Util::getChildNodeByName($mensagemRetorno, "Mensagem")->nodeValue;
                    return  array(
                        "CODIGO" => '2',
                        "DESCRICAO" => $mensagem,
                        "RPS"    => ''
                    );
                }
            }
        }
        else {
            return false;
        }
    }

    public function consultarLoteRps($lote) {
        $this->cdempresa = $lote["CDEMPLOTERPS"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $lote["CDINSCMUNI"]);

        $this->ambiente = Util::getAmbiente($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCMUNI"], $this->environment->getNrOrg());
        if ($this->ambiente == null) $this->ambiente = 2;

        //$this->RazaoSocialPrestador = $lote["NMRAZSOCEMPR"];
        $inscricaoJuridica = str_replace(array("-", "/", ".", " "), "", $lote["NRINSJURIEST"]);
        $inscricaoMunicipal = str_replace(array("-", "/", ".", " "), "", $lote["CDINSCMUNI"]);
        
        
        $xml = $this->gerarXmlConsultarSituacao($inscricaoJuridica, $inscricaoMunicipal, $lote["NRPROTOCOLORPS"]);
        $statusLote = $this->consultarSituacaoLoteRpsWS($xml, $lote["CDEMPLOTERPS"], $lote["NRINSJURIEST"], $lote["NRSEQLOTERPS"]);

        if(is_array($statusLote)) {
            $serieRps = $lote["CDSERIERPS"];
            
            $chave = "NUMERORPS".$lote["CDEMPLOTERPS"].$lote["NRINSJURIEST"].$lote["CDSERIERPS"];
            $nrRps = $this->newCode->getCodeByTableName($chave, $this->environment->getNrOrg(), 1, 15);
            $qtdRps = $lote["QTDERPS"];
            
            $nrRps = $nrRps - ($qtdRps + 1);
            
            Util::atualizarSituacaoNumeroRpsPeloLote($this->connection, $serieRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $this->environment->getNrOrg(), $this->environment->getCdOperador());
    
            Util::atualizarLote($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 3, $lote["NRPROTOCOLORPS"], $this->environment->getNrOrg(), $this->environment->getCdOperador());

            
            $erros[] = $statusLote;
            return $erros;
        }
        else if($statusLote == "4" || $statusLote == "3") {

            $xml = $this->gerarXmlConsultarLoteRps($inscricaoJuridica, $inscricaoMunicipal, $lote["NRPROTOCOLORPS"]);
            $resultado = $this->consultarLoteRpsWS($xml, $lote["CDEMPLOTERPS"], $lote["NRINSJURIEST"], $lote["NRSEQLOTERPS"]);

            if (is_string($resultado)) {
                $domResult = new \DOMDocument("1.0", "UTF-8");
                $domResult->loadXML($resultado);
                
                if ($domResult->getElementsByTagName("ListaNfse")->length > 0) {
       
                    $nodeList = $domResult->getElementsByTagName("InfNfse");

                    if ($nodeList->length > 0) {
                        foreach ($nodeList as $nota) {
                            $numeroNota = Util::getChildNodeByName($nota, "tc:Numero")->nodeValue;
                            $numeroRps = str_pad($numeroNota, 9, "0", STR_PAD_LEFT);
                            $codigoVerificacao = Util::getChildNodeByName($nota, "tc:CodigoVerificacao")->nodeValue;
                            $nodeIdentificacaoRps = $nota->getElementsByTagName("IdentificacaoRps")->item(0);
                            $serieRps = Util::getChildNodeByName($nodeIdentificacaoRps, "tc:Serie")->nodeValue;
                            $numeroRps = Util::getChildNodeByName($nodeIdentificacaoRps, "tc:Numero")->nodeValue;
                            $numeroRps = str_pad($numeroRps, 15, "0", STR_PAD_LEFT);
                           
                            /*==================================*/
                            $dataEmissao = Util::getChildNodeByName($nota, "tc:DataEmissao")->nodeValue;
                            $dataEmissao = date_create_from_format("Y-m-d\TH:i:s", $dataEmissao);
                            $dataEmissao = $dataEmissao->format("d/m/Y");
        
                            $competencia =  Util::getChildNodeByName($nota, "tc:Competencia")->nodeValue;
                            $competencia = date_create_from_format("Ym", $competencia);
                            $competencia = $competencia->format("m/Y");
                            $competencia = '01/'.$competencia;
                            /*==================================*/
                           
                            Util::atualizarNumeroRps($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], $numeroNota, $codigoVerificacao, 4, $dataEmissao, $competencia, $this->environment->getNrOrg(), $this->environment->getCdOperador());
        
                            Util::atualizarNotaConsulta($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], $numeroNota, $this->environment->getNrOrg(), $this->environment->getCdOperador());
        
                            Util::inserirArquivoXml($this->connection, $serieRps, $numeroRps, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], "A", $resultado, $this->environment->getNrOrg(), $this->environment->getCdOperador());
                        }
        
                        Util::atualizarLote($this->connection, $lote["CDEMPLOTERPS"], $lote["CDINSCESTA"], $lote["CDINSCMUNI"], $lote["NRSEQLOTERPS"], 4, $lote["NRPROTOCOLORPS"], $this->environment->getNrOrg(), $this->environment->getCdOperador());
        
                        return true;
                    }
                    else {
                        return null;
                    }
                }
            }
            else {
                return false;
            }
        }
        else {
            if($statusLote == '1') $descricaoErro = 'Lote não foi recebido!';
            else if($statusLote == '2') $descricaoErro = 'Lote em processamento. Tente novamente mais tarde!';
            else {
                $descricaoErro  = 'Não foi possível consultar o status do lote tente novamente mais tarde!';
            }
            $erros[] = array(
                "CODIGO" => '2',
                "DESCRICAO" => $descricaoErro,
                "RPS"    => $lote["NRSEQLOTERPS"]
            );
            
            return $erros;
        }
    }
    
    private function gerarXmlCancelarNota($nrnotafisc, $inscricaoJuridica, $inscricaoMunicipal, $codMotivoCancelamento, $cdmunicibge) {
        $this->xmlCancelarNota = '<p1:CancelarNfseEnvio xmlns:p1="http://www.issnetonline.com.br/webserviceabrasf/vsd/servico_cancelar_nfse_envio.xsd" xmlns:tc="http://www.issnetonline.com.br/webserviceabrasf/vsd/tipos_complexos.xsd" xmlns:ts="http://www.issnetonline.com.br/webserviceabrasf/vsd/tipos_simples.xsd">
            <Pedido>
                <tc:InfPedidoCancelamento>
                    <tc:IdentificacaoNfse>
                        <tc:Numero>'.$nrnotafisc.'</tc:Numero> 
                        <tc:Cnpj>'.$inscricaoJuridica.'</tc:Cnpj>
                        <tc:InscricaoMunicipal>'.$inscricaoMunicipal.'</tc:InscricaoMunicipal>
                        <tc:CodigoMunicipio>'.$cdmunicibge.'</tc:CodigoMunicipio>
                    </tc:IdentificacaoNfse>
                    <tc:CodigoCancelamento>'.$codMotivoCancelamento.'</tc:CodigoCancelamento>
                </tc:InfPedidoCancelamento>
            </Pedido>
        </p1:CancelarNfseEnvio>';
    }

    private function getXmlAssinadoCancelarNota() {
        $this->xmlCancelarNota = str_replace("&", "&#38;", $this->xmlCancelarNota);

        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($this->xmlCancelarNota, LIBXML_NOBLANKS);
        $domXml = $this->removerTagsVazias($domXml);
        $domXml->formatOutput = true;
        $xmlTemp = $domXml->saveXML($domXml->documentElement);

        $domXml = new \DOMDocument("1.0", "UTF-8");
        $domXml->loadXML($xmlTemp);
        $domXml->encoding = "UTF-8";
        $domXml = $this->assinaXml($domXml, array("InfPedidoCancelamento"));
        if ($domXml === false) {
            throw new \Exception("Certificado digital não encontrado!");
        }
        else {
            return $domXml->saveXML($domXml->documentElement);
        }
    }

    public function cancelarNotas($notas, $codMotivo, $motivo, $cdmunicibge, $cdmunicsiafi) {
        $this->cdempresa = $notas[0]["CDEMPRESA"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $notas[0]["CDINSCMUNI"]);

        $this->ambiente = Util::getAmbiente($this->connection, $notas[0]["CDEMPRESA"], $notas[0]["CDINSCMUNI"], $this->environment->getNrOrg());
        if ($this->ambiente == null) $this->ambiente = 2;

        $this->senhaCertificadoDigital = Util::getSenhaCertificadoDigital($this->connection, $notas[0]["CDEMPRESA"], $notas[0]["CDINSCMUNI"], $this->environment->getNrOrg());

        //$this->RazaoSocialPrestador = $notas[0]["NMRAZSOCEMPR"];
        $errosTotais = array();

        foreach ($notas as $nota) {
            $inscricaoJuridica = str_replace(array("-", "/", ".", " "), "", $nota["NRINSJURIEST"]);
            $inscricaoMunicipal = str_replace(array("-", "/", ".", " "), "", $nota["CDINSCMUNI"]);
            $this->gerarXmlCancelarNota($nota["NRNOTAFISC"], $inscricaoJuridica, $inscricaoMunicipal, $codMotivo, $cdmunicibge);
            $xml = $this->getXmlAssinadoCancelarNota();

            $resultado = $this->cancelarNotaWS($xml, $nota["CDEMPRESA"], $nota["NRINSJURIEST"], $nota["NRNOTAFISC"]);

            if (is_string($resultado)) {
                $domResult = new \DOMDocument("1.0", "UTF-8");
                $domResult->loadXML($resultado);
                $nodeList = $domResult->getElementsByTagName("MensagemRetorno");
                if ($nodeList->length > 0) {
                    $errosNota = array();
                    foreach ($nodeList as $erro) {
                        $errosNota[] = array(
                            "CODIGO" => Util::getChildNodeByName($erro, "Codigo")->nodeValue,
                            "DESCRICAO" => Util::getChildNodeByName($erro, "Mensagem")->nodeValue
                        );
                    }
                    $errosTotais[$nota["NRLANCTONF"]] = $errosNota;
                }
                else {
                    $motivos = array(
                        1 => "Erro na emissão",
                        2 => "Serviço não prestado"
                    );
                    $descMotivo = strtr($codMotivo, $motivos);

                    Util::inserirNotaCancelada($this->connection, "S", "C", $descMotivo, "C", $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);

                    Util::atualizarNotaCancelada($this->connection, "N", $nota["CDFILIAL"], $nota["NRLANCTONF"], $this->environment->getNrOrg());
                }
            }
            else {
                return false;
            }
        }
        return (count($errosTotais) > 0) ? $errosTotais : true;
    }

    private function cancelarNotaWS($xml, $cdempresa, $inscricao, $nrnotafisc) {
        $nomeFuncao = "CancelarNfse";
        $prefixoArquivo = $cdempresa.$inscricao.$nrnotafisc."_".$nomeFuncao."_";
        $nomeArquivoEnvio = $prefixoArquivo."env.xml";
        //file_put_contents($this->caminhoXml . $nomeArquivoEnvio, $xml);

        try {
            $resultado = $this->enviarParaWebservice($nomeFuncao, $xml);
        }
        catch (\Exception $ex) {
            throw new \Exception($ex->getMessage());
        }

        $nomeArquivoResposta = $prefixoArquivo."resp.xml";

        if ($resultado->outputXML) {
            //file_put_contents($this->caminhoXml . $nomeArquivoResposta, $resultado->outputXML);
            return $resultado->outputXML;
        }
        else {
            return false;
        }
    }

    private function enviarParaWebservice($nomeFuncao, $conteudo) {
        if($this->caminhoCertificado) {
            $caminho = $this->caminhoCertificado . $this->cdempresa . "_" . $this->cdinscmuni . DIRECTORY_SEPARATOR . "certKey.pem";
            $unlink  = false;
        }
        else {
            $certKey = $this->linkCertKey;
            $hash    = uniqid();
            if (is_dir("/tmp")) {
                $caminho= "/tmp/certKey$hash.pem";
            }
            else {
                $caminho = __DIR__ . DIRECTORY_SEPARATOR ."certKey$hash.pem";
            }
            file_put_contents($caminho, file_get_contents($certKey));
            $unlink = true;
        }
        
        $conteudo = str_replace("<", "&lt;", $conteudo);
        $conteudo = str_replace(">", "&gt;", $conteudo);
        
        $conteudo = $this->createSoapText($conteudo, $nomeFuncao);
        $soapVar = new \SoapVar($conteudo, XSD_ANYXML);
        $lotUri = self::URL_PRODUCAO;
        
        $resultado = $this->buildSoapCall(
            $caminho,
            $lotUri,
            $lotUri,
            $nomeFuncao,
            $soapVar,
            $conteudo
        );
        
        if($unlink) unlink($caminho);
        return $resultado;
    }
    
    public function createSoapText($conteudo, $nomeFuncao) {
        $conteudo = '<?xml version="1.0" encoding="utf-8"?><soap:Envelope
        	xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"
        	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        	xmlns:xsd="http://www.w3.org/2001/XMLSchema">
        	<soap:Body>
        		<'.$nomeFuncao.' xmlns="http://www.issnetonline.com.br/webservice/nfd">
        			<xml>&lt;?xml version="1.0" encoding="UTF-8"?&gt;'.$conteudo.'</xml>
        		</'.$nomeFuncao.'>
        	</soap:Body>
        </soap:Envelope>';
        return $conteudo;
        
    }

    public function buildSoapCall($cert, $location, $wsdl, $functionName, $soapVar, $xmlLote) {
       // var_dump($cert, $location, $wsdl, $functionName, $soapVar, $xmlLote);die;
        try {
            $msgSize = strlen($xmlLote);
            $parameters = [
                "Content-Type: text/xml;charset=UTF-8",
                "SOAPAction: \"http://www.issnetonline.com.br/webservice/nfd/$functionName\"",
                "Content-length: $msgSize"
            ];

            $oCurl = curl_init();
            curl_setopt($oCurl, CURLOPT_URL, $wsdl);
            curl_setopt($oCurl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
            curl_setopt($oCurl, CURLOPT_HEADER, 1);
            curl_setopt($oCurl, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($oCurl, CURLOPT_SSL_VERIFYPEER, 0);
            curl_setopt($oCurl, CURLOPT_SSLVERSION, 0);
            curl_setopt($oCurl, CURLOPT_SSLCERT, $cert);
            curl_setopt($oCurl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($oCurl, CURLOPT_POST, true);
            curl_setopt($oCurl, CURLOPT_POSTFIELDS, $xmlLote);
            curl_setopt($oCurl, CURLOPT_HTTPHEADER, $parameters);
            
            $response = curl_exec($oCurl);
            $this->soaperror = curl_error($oCurl);
            $ainfo = curl_getinfo($oCurl);
            if (is_array($ainfo)) {
                $this->soapinfo = $ainfo;
            }
            $headsize = curl_getinfo($oCurl, CURLINFO_HEADER_SIZE);
            $httpcode = curl_getinfo($oCurl, CURLINFO_HTTP_CODE);
            curl_close($oCurl);
            $responseHead = trim(substr($response, 0, $headsize));
            $responseBody = trim(substr($response, $headsize));
            if ($httpcode == 404) {
                throw new \Exception('Erro 404 ao comunicar com a Sefaz.', 1);
            } else if ($httpcode == 500) {
                throw new \Exception($responseBody, 1);
            } else if ($httpcode != 200) {
                throw new \Exception('Erro inesperado.', 1);
            } else {
                $inicioResponse = strpos($responseBody, "&lt;");
                $length         = strrpos($responseBody, "&gt;") - $inicioResponse + 4;
                $responseBody   = substr($responseBody, $inicioResponse, $length);
                $responseBody   = str_replace("&lt;", "<", $responseBody);
                $responseBody   = str_replace("&gt;", ">", $responseBody);
                $response       = new \stdClass();
                $response->outputXML = $responseBody;
                return $response;
            }

        } catch(\Exception $e) {
            throw $e;
        }
    }

	private function assinaXml(\DOMDocument $domXml, array $tags) {
	    $local = ($this->caminhoCertificado);
	    
        if($local) {
            $dir = $this->caminhoCertificado . $this->cdempresa . "_" . $this->cdinscmuni;

            if(!is_dir($dir)) return false;
            if((!is_file($dir . DIRECTORY_SEPARATOR . "certificado.pfx")) && (!is_file($dir . DIRECTORY_SEPARATOR . "certificado.p12"))) return false;
            else {
                if(is_file($dir . DIRECTORY_SEPARATOR . "certificado.pfx")) $data = file_get_contents($dir . DIRECTORY_SEPARATOR . "certificado.pfx");
                else if(is_file($dir . DIRECTORY_SEPARATOR . "certificado.p12")) $data = file_get_contents($dir . DIRECTORY_SEPARATOR . "certificado.p12");
            }
        }
        else {
            try{
                $data = file_get_contents($this->linkCertificado);
            }
            catch(\Exception $e) {
                return false;
            }
            if(empty($data)) return false;
        }
        
        openssl_pkcs12_read($data, $cert, $this->senhaCertificadoDigital);

        if (is_null($cert)) {
            return false;
        }
        else {
            //file_put_contents($dir.DIRECTORY_SEPARATOR."certKey.pem", $cert["pkey"]."\r\n".$cert["cert"]);

            $this->publicKey = $this->getPublicKey($cert["cert"]);
            $this->privateKeyId = openssl_get_privatekey($cert["pkey"]);

            foreach ($tags as $tag) {
                $domXml = $this->assinaXmlAux($domXml, $tag);
            }

            openssl_free_key($this->privateKeyId);
            return $domXml;
        }

    }

	private function assinaXmlAux(\DOMDocument $domXml, $tag) {
		$nodes = $domXml->getElementsByTagName($tag);
		foreach ($nodes as $node) {
			$root = $node->parentNode;
			$id = trim($node->getAttribute("Id"));
			$dados = $node->C14N(false, false, null, null);
			$hashValue = hash("sha1", $dados, true);
			$digValue = base64_encode($hashValue);

            $signature = $domXml->createElement("Signature");
            $root->appendChild($signature);
            $signature->setAttribute("xmlns", "http://www.w3.org/2000/09/xmldsig#");
            //$signature->setAttribute("Id", "Ass_".$id);

            $signedInfo = $domXml->createElement("SignedInfo");
	        $signature->appendChild($signedInfo);

	        $novaTag = $domXml->createElement("CanonicalizationMethod");
	        $signedInfo->appendChild($novaTag);
	        $novaTag->setAttribute("Algorithm", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");

	        $novaTag = $domXml->createElement("SignatureMethod");
	        $signedInfo->appendChild($novaTag);
	        $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#rsa-sha1");

	        $reference = $domXml->createElement("Reference");
	        $signedInfo->appendChild($reference);
	        $reference->setAttribute("URI", "#".$id);

	        $transforms = $domXml->createElement("Transforms");
	        $reference->appendChild($transforms);

	        $novaTag = $domXml->createElement("Transform");
	        $transforms->appendChild($novaTag);
	        $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#enveloped-signature");

	        $novaTag = $domXml->createElement("Transform");
	        $transforms->appendChild($novaTag);
	        $novaTag->setAttribute("Algorithm", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");

	        $novaTag = $domXml->createElement("DigestMethod");
	        $reference->appendChild($novaTag);
	        $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#sha1");

	        $novaTag = $domXml->createElement("DigestValue", $digValue);
	        $reference->appendChild($novaTag);

            $dados = $signedInfo->C14N(false, false, null, null);
            $assinatura = "";
            openssl_sign($dados, $assinatura, $this->privateKeyId);
            $signatureValue = base64_encode($assinatura);


	        $novaTag = $domXml->createElement("SignatureValue", $signatureValue);
	        $signature->appendChild($novaTag);

	        $keyInfo = $domXml->createElement("KeyInfo");
	        $signature->appendChild($keyInfo);

	        $X509Data = $domXml->createElement("X509Data");
	        $keyInfo->appendChild($X509Data);

	        $novaTag = $domXml->createElement("X509Certificate", $this->publicKey);
	        $X509Data->appendChild($novaTag);
    	}
        return $domXml;
	}

	private function getPublicKey($key) {
        $data = "";
        $arCert = explode("\n", $key);
        foreach ($arCert as $curData) {
            //remove a tag de inicio e fim do certificado
            if (strncmp($curData, "-----BEGIN CERTIFICATE", 22) != 0 &&
                    strncmp($curData, "-----END CERTIFICATE", 20) != 0 ) {
                //carrega o resultado numa string
                $data .= trim($curData);
            }
        }
        return $data;
    }

    private function gerarXmlConsultarLoteRps($inscricaoPrestador, $inscricaoMunicipalPrestador, $protocolo) {
        $this->xmlConsultarLoteRps = '<ConsultarLoteRpsEnvio xmlns="http://www.issnetonline.com.br/webserviceabrasf/vsd/servico_consultar_lote_rps_envio.xsd" xmlns:ts="http://www.issnetonline.com.br/webserviceabrasf/vsd/tipos_simples.xsd" xmlns:tc="http://www.issnetonline.com.br/webserviceabrasf/vsd/tipos_complexos.xsd">
            <Prestador>
                <tc:CpfCnpj>
					';
    				if (strlen($this->InscricaoPrestador) == 11) {
    					$this->xmlConsultarLoteRps .='<tc:Cpf>'.$inscricaoPrestador.'</tc:Cpf>';
    				}
    				else {
    					$this->xmlConsultarLoteRps .= '<tc:Cnpj>'.$inscricaoPrestador.'</tc:Cnpj>';
    				}
    				$this->xmlConsultarLoteRps .= '
				</tc:CpfCnpj>
                <tc:InscricaoMunicipal>'.$inscricaoMunicipalPrestador.'</tc:InscricaoMunicipal>
            </Prestador>
            <Protocolo>'.$protocolo.'</Protocolo>
        </ConsultarLoteRpsEnvio>';
        return $this->xmlConsultarLoteRps;
    }
    
    private function gerarXmlConsultarSituacao($inscricaoPrestador, $inscricaoMunicipalPrestador, $protocolo) {
        $this->xmlConsultarLoteRps = '<ConsultarSituacaoLoteRpsEnvio xmlns="http://www.issnetonline.com.br/webserviceabrasf/vsd/servico_consultar_situacao_lote_rps_envio.xsd" xmlns:tc="http://www.issnetonline.com.br/webserviceabrasf/vsd/tipos_complexos.xsd">
            <Prestador>
                <tc:CpfCnpj>
					';
    				if (strlen($this->InscricaoPrestador) == 11) {
    					$this->xmlConsultarLoteRps .='<tc:Cpf>'.$inscricaoPrestador.'</tc:Cpf>';
    				}
    				else {
    					$this->xmlConsultarLoteRps .= '<tc:Cnpj>'.$inscricaoPrestador.'</tc:Cnpj>';
    				}
    				$this->xmlConsultarLoteRps .= '
				</tc:CpfCnpj>
                <tc:InscricaoMunicipal>'.$inscricaoMunicipalPrestador.'</tc:InscricaoMunicipal>
            </Prestador>
            <Protocolo>'.$protocolo.'</Protocolo>
        </ConsultarSituacaoLoteRpsEnvio>';
        return $this->xmlConsultarLoteRps;
    }

    public function enviarLoteRps($notas) {
        $erros = array("TIPO" => 1);
        $this->listaAuxRps = array();

        $nrlanctonf = $notas[0]["NRLANCTONF"];
        $cdfilial = $notas[0]["CDFILIAL"];

        $notafiscal = Util::getCorpoNF($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
       
        $this->cdempresa = $notafiscal["CDEMPRESA"];
        $this->cdinscmuni = str_replace(array(".","-","/"," "), "", $notafiscal["CDINSCMUNIEMP"]);

        $this->ambiente = Util::getAmbiente($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $this->environment->getNrOrg());
        if ($this->ambiente == null) $this->ambiente = 2;

        $this->senhaCertificadoDigital = Util::getSenhaCertificadoDigital($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $this->environment->getNrOrg());
    
        $this->NumeroLote = Util::inserirLoteRps($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCESTA"], $notafiscal["CDINSCMUNIEMP"], $notafiscal["NRINSJURIEST"], $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);
        $this->IdLote = "LOTE_".$notafiscal["CDEMPRESA"]."_".$this->NumeroLote;
        $this->InscricaoPrestador = str_replace(array(".","-","/"), "", $notafiscal["NRINSJURIEST"]);
        $this->InscricaoMunicipalPrestador = str_replace(array(".","-","/"), "", $notafiscal["CDINSCMUNIEMP"]);
        $this->RazaoSocialPrestador = $notafiscal["NMRAZSOCEMIT"];
        
        if(!isset($dadosNfct[0])) {
            if($notafiscal["IDNATUOPEREMPR"] == 1) {
                $codigoMunicipoPrestador = (int)$notafiscal["CDMUNICIBGEEMP"];
            }
            else {
                
                $codigoMunicipoPrestador = (int)$notafiscal["CDMUNICIBGECLIE"];
                
            }
        }
        else {
            $codigoMunicipoPrestador = $dadosNfct[0]["CDMUNICIBGE"];
        }
        
        $this->CodigoMunicipioPrestador = $codigoMunicipoPrestador;

        foreach ($notas as $nota) {
            $nrlanctonf = $nota["NRLANCTONF"];
            $cdfilial = $nota["CDFILIAL"];
            $notafiscal = Util::getCorpoNF($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());

            //VALIDACAO
            if (empty($notafiscal["CDINSCMUNIEMP"])) {
                $erros[] = array(
                    "CODIGO" => $notafiscal["NRLANCTONF"],
                    "DESCRICAO" => "Inscrição municipal deve ser relacionada com a empresa informada."
                );
            }
            //VALIDACAO
            if ($notafiscal["IDNATUOPEREMPR"] == "99") {
                $erros[] = array(
                    "CODIGO" => $notafiscal["NRLANCTONF"],
                    "DESCRICAO" => "Natureza da operação deve ser informada."
                );
            }
            //VALIDACAO
            if ($notafiscal["IDREGESPTRIBEMP"] == "99") {
                $erros[] = array(
                    "CODIGO" => $notafiscal["NRLANCTONF"],
                    "DESCRICAO" => "Regime especial de tributação deve ser informado."
                );
            }

            $this->NaturezaOperacao = (int)$notafiscal["IDNATUOPEREMPR"];
            $this->RegimeEspecialTributacao = "";
            $this->OptanteSimplesNacional = 2;

            if ($notafiscal["IDOPTSIMPLES"] == "S") {
                $this->RegimeEspecialTributacao = (int)$notafiscal["IDREGESPTRIBEMP"];
                $this->OptanteSimplesNacional = 1;
            }
            $this->IncentivadorCultural = 2;

            $dataAtual = date_create();
            //$this->DataEmissao = $dataAtual->format("Y-m-d\TH:i:s");
            $this->DataEmissao = $notafiscal["DTENTRSAID"] . "T" . $dataAtual->format("H:i:s");
            //$this->DataEmissao = '09-08-2018 15:50:57'; 

            $this->NumeroRps = Util::inserirNumeroRps($this->connection, $this->NumeroLote, 2, $notafiscal["CDEMPRESA"], $notafiscal["CDSERIERPS"], null, $notafiscal["CDINSCESTA"], $notafiscal["CDINSCMUNIEMP"], $notafiscal["NRINSJURIEST"], null, null, $this->environment->getNrOrg(), $this->environment->getCdOperador(), $this->newCode);

            $this->SerieRps = $notafiscal["CDSERIERPS"];
            $this->IdRps = "RPS_" . $this->SerieRps . "_" . $this->NumeroRps;
            $this->TipoRps = 1;
            $this->StatusRps = 1;

            $this->InscricaoTomador = str_replace(array(".","-","/"," "), "", $notafiscal["NRINSJURCLIE"]);
            $this->InscricaoMunicipalTomador = str_replace(array(".","-","/"," "), "", $notafiscal["NRINSMUNICLIE"]);
            if (strtoupper($this->InscricaoMunicipalTomador) == "ISENTO" || strtoupper($this->InscricaoMunicipalTomador) == "ISENTA") {
                $this->InscricaoMunicipalTomador = "";
            }
            $this->RazaoSocialTomador = $notafiscal["NMRAZSOCCLIE"];
            $this->Endereco = substr($notafiscal["DSENDECLIE"], 0, 125);
            $this->Numero = ".";
            $this->Complemento = substr($notafiscal["DSCOMPENDCLI"], 0, 60);
            $this->Bairro = substr($notafiscal["NMBAIRCLIE"], 0, 60);
            $this->CodigoMunicipioTomador = $notafiscal["CDMUNICIBGECLIE"];
            $this->Uf = $notafiscal["SGESTADOCLIE"];
            $this->Cep = substr(str_replace(array(".","-"), "", $notafiscal["NRCEPCLIE"]), 0, 8);
            $this->Telefone = substr(str_replace(array("(", ")", "-", " "), "", $notafiscal["NRTELECLIE"]), 0, 11);
            $this->Email = substr($notafiscal["DSEMAILCLIENFE"], 0, 80);

            $this->ValorServicos = $notafiscal["VRPRODUTOS"];
            $valorLiquido = $notafiscal["VRPRODUTOS"];
            $this->ValorDeducoes = 0.0;
            $this->ValorPis = 0.0;
            $this->ValorCofins = 0.0;
            $this->ValorInss = 0.0;
            $this->ValorIr = 0.0;
            $this->ValorCsll = 0.0;
            $this->IssRetido = 2;
            $this->ValorIss = 0.0;
            $this->ValorIssRetido = 0.0;
            $this->OutrasRetencoes = 0.0;
            $this->DescontoCondicionado = 0.0;
            $this->DescontoIncondicionado = 0.0;
            $this->BaseCalculo = 0.0;

            $impostos = Util::getImpostos($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());

            foreach ($impostos as $imp) {
                if (in_array($imp["CDIMPOSTONFE"], array("50", "55", "56")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorPis = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorPis;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("60", "65", "66")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorCofins = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorCofins;
                }
                else if ($imp["CDIMPOSTONFE"] == "42" && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorInss = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorInss;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("22", "25")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorIr = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorIr;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("70", "75", "76")) && $imp["IDTRATIMPO"] == "R") {
                    $this->ValorCsll = (float)$imp["VALORIMPOSTO"];
                    $valorLiquido -= $this->ValorCsll;
                }
                else if (in_array($imp["CDIMPOSTONFE"], array("31", "35", "36"))) {
                    if ($imp["IDTRATIMPO"] == "R") {
                        $this->IssRetido = 1;
                        $this->ValorIssRetido = (float)$imp["VALORIMPOSTO"];
                        $valorLiquido -= $this->ValorIssRetido;
                    }
                    else {
                        $this->ValorIss = (float)$imp["VALORIMPOSTO"];
                        $this->BaseCalculo = $imp["BASECALCULO"];
                    }
                }
            }

            $outrasRetencoes = Util::getOutrasRetencoes($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            if (!empty($outrasRetencoes["VRIMPOSTOIT"]) && $outrasRetencoes["VRIMPOSTOIT"] > 0) {
                $this->OutrasRetencoes = $outrasRetencoes["VALORIMPOSTO"];
                $valorLiquido -= $this->OutrasRetencoes;
            }

            $this->ValorLiquidoNfse = $valorLiquido;

            if (count($impostos) > 0) {
                $impostoItem = Util::getImpostoItemMaiorAliquota($this->connection, $nrlanctonf, $cdfilial, "31", $this->environment->getNrOrg());
                $this->Aliquota = 0;
                if (!empty($impostoItem)) {
                    $this->Aliquota = (float)$impostoItem["VRALIQUOTA"] * 100;
                    $this->Aliquota = number_format($this->Aliquota, 3);
                }
            }
            else {
                //VALIDACAO
                $erros[] = array(
                    "CODIGO" => $nota["NRLANCTONF"],
                    "DESCRICAO" => "Impostos da nota devem ser informados."
                );
            }

            $itemProduto = Util::getItemProduto($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            $cdarvprod = $itemProduto[0]["CDARVPROD"];

            $descricaoProd = "";

            foreach ($itemProduto as $linha) {
                $descricaoProd .= $linha["DSPRODEMIS"] . "\r\n";
            }

            $itemEncargo = Util::getItemEncargo($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            //$descricaoEncargo = "";
            foreach ($itemEncargo as $item) {
                //$descricaoEncargo .= $item["DESCRICAO"] . "\n";
                if ($item["IDENCARGO"] == "D") {
                    $this->DescontoIncondicionado += (float)$item["VRITEMENCA"];
                }
            }

            /*$descricaoDadosAdic = "";
            $dadosAdicionais = Util::getDadosAdicionais($this->connection, $nrlanctonf, $cdfilial, $this->environment->getNrOrg());
            foreach ($dadosAdicionais as $item) {
                $descricaoDadosAdic .= $item["DSDADOADIC"];
            }

            $this->Discriminacao = $descricaoProd;
            if (!empty($descricaoEncargo)) {
                $this->Discriminacao .= "|" . $descricaoEncargo;
            }
            if (!empty($descricaoDadosAdic)) {
                $this->Discriminacao .= "|" . $descricaoDadosAdic;
            }*/

            $this->Discriminacao = trim($descricaoProd);
            //$this->Discriminacao = substr(Util::retiraAcento($this->Discriminacao), 0, 2000);
            $this->Discriminacao = substr($this->Discriminacao, 0, 2000);
            $this->CodigoCnae = str_replace(array("-", "/", ".", " "), "", $notafiscal["CDCNAE"]);
            
            //var_dump( $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $cdarvprod, $this->environment->getNrOrg());
            $codTribMuni = Util::getCodTribMuni($this->connection, $notafiscal["CDEMPRESA"], $notafiscal["CDINSCMUNIEMP"], $cdarvprod, $this->environment->getNrOrg());
            //VALIDACAO
            if (!is_array($codTribMuni) || empty($codTribMuni["CDCLASSERVPADR"])) {
                $erros[] = array(
                    "CODIGO" => $notafiscal["NRLANCTONF"],
                    "DESCRICAO" => "Código da classificação do serviço deve ser informado."
                );
            }
            else {
                $this->ItemListaServico = $codTribMuni["CDCLASSERVPADR"];//1.01
                $this->CodigoTributacaoMunicipio = $codTribMuni["CDTRIBPRODPADR"];
            }

            $this->adicionarRpsEnviarLote();

            $this->listaAuxRps[] = array(
                "CDSERIERPS"   => $notafiscal["CDSERIERPS"],
                "NRRPS"        => $this->NumeroRps,
                "CDEMPLOTERPS" => $notafiscal["CDEMPRESA"],
                "CDINSCESTA"   => $notafiscal["CDINSCESTA"],
                "CDINSCMUNI"   => $notafiscal["CDINSCMUNIEMP"],
                "NRSEQLOTERPS" => $this->NumeroLote,
                "NRINSJURIEST" => $notafiscal["NRINSJURIEST"],
                "CDFILIAL"     => $cdfilial,
                "NRLANCTONF"   => $nrlanctonf
            );
        }

        if (count($erros) > 1) {
            $this->limparListaRps();
            return $erros;
        }
        else {
            $this->gerarXmlEnviarLoteRps();
            $xml = $this->getXmlAssinadoEnviarLoteRps();
            return $this->enviarLoteRpsWS($xml, $notafiscal["CDEMPRESA"], $notafiscal["NRINSJURIEST"], $this->NumeroLote);
        }
    }

}